@@ -3,6 +3,8 @@ from collections import OrderedDict
 from dataclasses import dataclass
 from typing import Any, List, Union
 
+from apgw.types import Params
+
 
 @dataclass(frozen=True)
 class BinaryConstraint:
@@ -14,7 +16,7 @@ class BinaryConstraint:
 @dataclass(frozen=True)
 class TextConstraint:
     sql: str
-    params: List[Any]
+    params: Params
 
 
 class DictConstraint(OrderedDict):
@@ -37,10 +39,14 @@ class GroupConstraint:
         self.constraints.append(constraint)
 
 
-Constraints = Union[
+Assignments = Union[
     BinaryConstraint,
     DictConstraint,
     List[BinaryConstraint],
     TextConstraint,
+]
+
+Constraints = Union[
+    Assignments,
     GroupConstraint,
 ]
