@@ -1,28 +1,22 @@
 """Contains the DB wrapper class."""
 from contextlib import asynccontextmanager
-from logging import Logger
-from typing import Any, AsyncIterator, Callable, Iterable, List, Optional, cast
+from time import time
+from typing import Any, AsyncIterator, Iterable, List, Optional, cast
 
 from asyncpg import Connection, connect
 from asyncpg.transaction import Transaction
 
 import apgw.builder as builder
-from apgw.constraint import Constraints
-from apgw.types import (ConnectArgs, ConnectionArgs, InsertAssignments,
-                        LimitOffset, Record, RecordsAndCount)
+from apgw.constraint import Assignments, Constraints
+from apgw.types import (ConnectionArgs, InsertAssignments, LimitOffset, Params,
+                        Record, RecordsAndCount)
 
 
 class DBBase:
     """A simple wrapper around asyncpg to alleviate some boilerplate."""
 
     def __init__(self) -> None:
-        """Initializes the DB instance.
-
-        :param logger: a Logger instance
-        :param connection: method to connect to the database
-        """
-        self.logger: Optional[Logger] = None
-        self.connect = connect
+        """Initializes the DB instance. """
         self._conn: Optional[Connection] = None
         self._current_transaction: Optional[Transaction] = None
 
@@ -47,6 +41,11 @@ class DBBase:
         """Returns True if the connection can and should be opened automatically."""
         return not self.is_open
 
+    def log(self, sql: str, params: Optional[Params], duration: float) -> None:
+        """Log out sql and params.
+        Override to do something with this information.
+        """
+
     async def auto_open(self) -> bool:
         """Opens the database if it should auto open."""
         if self.should_auto_open:
@@ -54,27 +53,20 @@ class DBBase:
             return True
         return False
 
-    def set_logger(self, logger: Optional[Logger]) -> "DBBase":
-        self.logger = logger
-        return self
-
-    def set_connect(
-        self, custom_connect: Callable[[ConnectArgs], Connection]
-    ) -> "DBBase":
-        self.connect = custom_connect
-        return self
-
     async def get_connection_args(self) -> ConnectionArgs:
         raise NotImplementedError()
 
+    async def connect(self) -> "Connection":
+        """Override this method to customize how the connection is open. For instance you could
+        use a connection pool.
+        """
+        conn_args = await self.get_connection_args()
+        return await connect(**conn_args)
+
     async def open(self) -> "DBBase":
         """Opens the connection."""
         if not self._conn:
-            conn_args = await self.get_connection_args()
-            if isinstance(conn_args, str):
-                self._conn = await self.connect(dsn=conn_args)
-            else:
-                self._conn = await self.connect(**conn_args)
+            self._conn = await self.connect()
         return self
 
     async def close(self) -> "DBBase":
@@ -107,44 +99,45 @@ class DBBase:
         """Asynchronous context manager exit point."""
         await self.close()
 
-    def _log(self, sql: str, params: List[Any] = None) -> None:
-        """Log out sql and params."""
-        if self.logger:
-            self.logger.info("SQL: %s", sql)
-            if params:
-                self.logger.info("PARAMS: %s", params)
-
-    async def execute(self, sql: str, params: List[Any] = None):
+    async def execute(self, sql: str, params: Optional[Params] = None):
         """This method passes through to self.conn.execute.
 
         :param sql: the parameterized sql to execute
         :param params: a list of params
         """
-        self._log(sql, params)
 
         await self.auto_open()
 
+        start = time()
+
         if params:
             await self.conn.execute(sql, *params)
         else:
             await self.conn.execute(sql)
 
-    async def executemany(self, sql: str, params: List[Iterable[Any]] = None):
+        self.log(sql, params, time() - start)
+
+    async def executemany(self, sql: str, params: Optional[List[Iterable[Any]]] = None):
         """This method passes through to self.conn.execute.
 
         :param sql: the parameterized sql to execute
         :param params: a list of params
         """
-        self._log(sql, params)
 
         await self.auto_open()
 
+        start = time()
+
         if params:
             await self.conn.executemany(sql, params)
         else:
             await self.conn.executemany(sql)
 
-    async def fetch(self, sql: Optional[str], params: List[Any] = None) -> List[Record]:
+        self.log(sql, params, time() - start)
+
+    async def fetch(
+        self, sql: Optional[str], params: Optional[Params] = None
+    ) -> List[Record]:
         """This method passes through to self.conn.fetch.
 
         :param sql: the parameterized sql to execute
@@ -153,17 +146,20 @@ class DBBase:
         if not sql:
             raise Exception("No SQL provided")
 
-        self._log(sql, params)
-
         await self.auto_open()
 
-        if params:
-            return await self.conn.fetch(sql, *params)
+        try:
+            start = time()
 
-        return await self.conn.fetch(sql)
+            if params:
+                return await self.conn.fetch(sql, *params)
+
+            return await self.conn.fetch(sql)
+        finally:
+            self.log(sql, params, time() - start)
 
     async def fetchrow(
-        self, sql: Optional[str], params: List[Any] = None
+        self, sql: Optional[str], params: Optional[Params] = None
     ) -> Optional[Record]:
         """This method passes through to self.conn.fetchrow.
 
@@ -173,23 +169,26 @@ class DBBase:
         if not sql:
             raise Exception("No SQL provided")
 
-        self._log(sql, params)
-
         await self.auto_open()
 
-        if params:
-            return await self.conn.fetchrow(sql, *params)
+        start = time()
 
-        return await self.conn.fetchrow(sql)
+        try:
+            if params:
+                return await self.conn.fetchrow(sql, *params)
+
+            return await self.conn.fetchrow(sql)
+        finally:
+            self.log(sql, params, time() - start)
 
     async def select(
         self,
         table_name: str,
-        constraints: Constraints = None,
+        constraints: Optional[Constraints] = None,
         *,
         columns: str = "*",
-        order_by: str = None,
-        limit: LimitOffset = None,
+        order_by: Optional[str] = None,
+        limit: Optional[LimitOffset] = None,
     ) -> List[Record]:
         """Selects records from the database.
 
@@ -212,11 +211,11 @@ class DBBase:
     async def select_and_count(
         self,
         table_name: str,
-        constraints: Constraints = None,
+        constraints: Optional[Constraints] = None,
         *,
         columns: str = "*",
-        order_by: str = None,
-        limit: LimitOffset = None,
+        order_by: Optional[str] = None,
+        limit: Optional[LimitOffset] = None,
     ) -> RecordsAndCount:
         """Counts records that match a constraint and then returns a limited set of them
 
@@ -247,10 +246,10 @@ class DBBase:
     async def select_one(
         self,
         table_name: str,
-        constraints: Constraints = None,
+        constraints: Optional[Constraints] = None,
         *,
         columns: str = "*",
-        order_by: str = None,
+        order_by: Optional[str] = None,
     ) -> Optional[Record]:
         """Selects a single record from the database.
 
@@ -269,7 +268,9 @@ class DBBase:
 
         return await self.fetchrow(sql, params)
 
-    async def exists(self, table_name: str, constraints: Constraints = None) -> bool:
+    async def exists(
+        self, table_name: str, constraints: Optional[Constraints] = None
+    ) -> bool:
         """Returns True if the record exists, False otherwise.
 
         :param table_name: the name of the table to select from
@@ -280,7 +281,11 @@ class DBBase:
         return await self.fetchrow(sql, params) is not None
 
     async def insert(
-        self, table_name: str, assignments: InsertAssignments, *, returning: str = None
+        self,
+        table_name: str,
+        assignments: InsertAssignments,
+        *,
+        returning: Optional[str] = None,
     ) -> Record:
         """Inserts a record into the database.
 
@@ -310,11 +315,11 @@ class DBBase:
     async def update(
         self,
         table_name: str,
-        assignments: Constraints,
-        constraints: Constraints = None,
+        assignments: Assignments,
+        constraints: Optional[Constraints] = None,
         *,
         timestamp_col: Optional[str] = "updated_at",
-        returning: str = None,
+        returning: Optional[str] = None,
     ) -> List[Record]:
         """Updates records in the database.
 
@@ -335,7 +340,11 @@ class DBBase:
         return await self.fetch(sql, params)
 
     async def delete(
-        self, table_name: str, constraints: Constraints = None, *, returning: str = None
+        self,
+        table_name: str,
+        constraints: Optional[Constraints] = None,
+        *,
+        returning: Optional[str] = None,
     ) -> List[Record]:
         """Deletes records from the database.
 
@@ -356,15 +365,14 @@ class DB(DBBase):
 
         :param conn_args: the connection arguments
         :param auto_open: if the connection should automatically open
-        :param logger: a Logger instance
         """
         super().__init__()
-        self.conn_args = conn_args
+        self._conn_args = conn_args
         self._auto_open = auto_open
 
     @property
     def should_auto_open(self) -> bool:
         return self._auto_open and not self.is_open
 
-    async def get_connection_args(self) -> Connection:
-        return self.conn_args
+    async def get_connection_args(self) -> ConnectionArgs:
+        return self._conn_args
