@@ -2,7 +2,7 @@
 from typing import Any, List
 
 from apgw.constraint import BinaryConstraint, Literal
-from apgw.types import SQL
+from apgw.types import SQL, Params
 
 
 def constraint_list_to_sql(
@@ -15,7 +15,7 @@ def constraint_list_to_sql(
 ) -> SQL:
     """Turns a list of constraints into a (sql, params) pair."""
     parts: List[str] = []
-    params: List[Any] = []
+    params: Params = []
     placeholder = 1
 
     for constraint in constraints:
